#
# Copyright 2012 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Implementation of Inspector abstraction for pcenter."""

from lxml import etree
from oslo_config import cfg
from oslo_utils import units
import six
import time

from ceilometer.compute.virt.pcenter import client
from ceilometer.compute.pollsters import util
from ceilometer.compute.virt import inspector as virt_inspector
#from ceilometer.i18n import _
from ceilometer.openstack.common.gettextutils import _
from ceilometer.openstack.common import log as logging


LOG = logging.getLogger(__name__)

opt_group = cfg.OptGroup(name='pcenter',
                         title='Options for pcenter')

OPTS = [
    cfg.StrOpt('pcenter_name',
               default='ceilometer',
               help='pcenter username.'),
    cfg.StrOpt('pcenter_passwd',
               default='admin',
               help='pcenter passwd.'),
    cfg.StrOpt('pcenter_url',
               default='http://192.168.137.233:8080/rest',
               help='Override the default pcenter URI '
                    '(which is dependent on pcenter_type).'),
]
CONF=cfg.CONF
CONF.register_group(opt_group)
CONF.register_opts(OPTS, group=opt_group)

class PcenterInspector(virt_inspector.Inspector):


    def __init__(self):
        super(PcenterInspector, self).__init__()
        self.passwd = CONF.pcenter.pcenter_passwd
        self.url = CONF.pcenter.pcenter_url
        self.user = CONF.pcenter.pcenter_name
        self.client = client.PcenterClient(self.url,
                                           self.user,
                                           self.passwd,
                                           None,
                                           None)

    def get_monitor(self, instance,resource):
        """Get the current status of an instance, by name (not ID!)
        Returns a InstanceInfo object
        """
        instance_name = instance.name
        path = ('/vm/%(display_name)s/%(resource)s/monitor'% {
                    'display_name': instance_name,
                    'resource':resource})
        response = self.client.get(path)
        status_code = response.status_code
        if status_code != 200 :
            msg = (_(
                'Error from pcenter while get instance %(instance_uuid)s info.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'instance_uuid': instance['uuid'],
                                          'status_code': status_code,
                                          'reason': response.reason}))
            raise virt_inspector.InspectorException(msg)
        result = response.json()
        if result == None:
            raise virt_inspector.InspectorException('Response error')
        lpar_state = result.get('status')
        if result.get('code') == "1001" or result.get('code') == "1002":
            raise virt_inspector.InspectorException('%s' %(result.get('msg')))
        if result.get('code') == "3006":
            raise virt_inspector.InspectorException('this vm not found')
        if result.get('code') == "0001":
            raise virt_inspector.InspectorException('%s' %(result.get('msg')))
        return result

    def get_resource(self, instance,resource):
        """Get the current status of an instance, by name (not ID!)
        Returns a InstanceInfo object
        """
        instance_name = instance.name
        path = ('/vm/%(display_name)s/%(resource)s/resource'% {
                    'display_name': instance_name,
                    'resource':resource})
        response = self.client.get(path)
        status_code = response.status_code
        if status_code != 200 :
            msg = (_(
                'Error from pcenter while get instance %(instance_uuid)s info.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'instance_uuid': instance['uuid'],
                                          'status_code': status_code,
                                          'reason': response.reason}))
            raise virt_inspector.InspectorException(msg)
        result = response.json()
        if result == None:
            raise virt_inspector.InspectorException('Response error')
        lpar_state = result.get('status')
        if result.get('code') == "1001" or result.get('code') == "1002":
            raise virt_inspector.InspectorException('%s' %(result.get('msg')))
        if result.get('code') == "3006":
            raise virt_inspector.InspectorException('this vm not found')
        if result.get('code') == "0001":
            raise virt_inspector.InspectorException('%s' %(result.get('msg')))
        return result

    def inspect_cpu_util(self,instance,duration=None):
        info = self.get_monitor(instance,'cpu')
        utils = info['cpu_rate']
        return virt_inspector.CPUUtilStats(util=utils)

    def inspect_cpus(self, instance):
        '''get the count and runtime for cpu by the 
        name of instance'''

        info = self.get_resource(instance,'cpu')
        number=float(info['cpu_core'])
        run_time=info['runTime']
        cpu_time = run_time * 60 * 1000 * 1000 * 1000
        return virt_inspector.CPUStats(number=number, time=cpu_time)

    def inspect_memory_resident(self, instance, duration=None):
        info = self.get_resource(instance,'memory')
        memory = info['memory_total']
        return virt_inspector.MemoryResidentStats(resident=memory)

    def inspect_memory_usage(self, instance, duration=None):
        info = self.get_monitor(instance,'memory')
        memory_used = info['memory_used']
        return virt_inspector.MemoryUsageStats(usage=memory_used)

    def inspect_disks(self, instance):
        info = self.get_monitor(instance,'disk')
        for key in info.keys():
            if key == 'code' or key == 'msg':
	        continue
	    device = key
            disk = virt_inspector.Disk(device=device)
	    read_requests = 0
	    write_requests = 0
	    read_bytes = info[key]['kb_r_total'] * units.Ki
	    write_bytes = info[key]['kb_w_total'] * units.Ki
            stats = virt_inspector.DiskStats(read_requests=read_requests,
		                             read_bytes=read_bytes,
		                             write_requests=write_requests,
		                             write_bytes=write_bytes,
		                             errors=0)
            yield (disk,stats)

    def inspect_disk_info(self, instance):
        info = self.get_resource(instance,'disk')
        for key in info.keys():
            if key == 'code' or key == 'msg':
                continue
            device = key
            total_disk = info[key]['disk_total_size']
            used_disk = info[key]['disk_used_size']
            if total_disk == 0:
                usage = 0
            else:
                usage = used_disk / total_disk
            disk = virt_inspector.Disk(device=device)
            disk_info = virt_inspector.DiskInfo(capacity=total_disk,
                                               allocation=used_disk,
                                               physical=usage)
            yield (disk,disk_info)

    def inspect_disk_iops(self,instance):
        info = self.get_monitor(instance,'disk')
        for key in info.keys():
            if key == 'code' or key == 'msg':
                continue
            
            iops = info[key]['tps_r'] + info[key]['tps_w']
            device = key
            disk = virt_inspector.Disk(device=device)
            disk_iops = virt_inspector.DiskIOPSStats(iops_count=iops)
            yield (disk,disk_iops)

    def inspect_disk_rates(self,instance,duration=None):
        info = self.get_monitor(instance,'disk')
        for key in info.keys():
            if key == 'code' or key == 'msg':
                continue
            device = key
            disk = virt_inspector.Disk(device=device)
            read_bytes_rate = info[key]['kb_r'] * units.Ki
            write_bytes_rate = info[key]['kb_w'] * units.Ki
            read_requests_rate = info[key]['tps_r']
            write_requests_rate = info[key]['tps_w']
            disk_rate_info = virt_inspector.DiskRateStats(read_bytes_rate=read_bytes_rate,
                                                          write_bytes_rate=write_bytes_rate,
                                                          read_requests_rate=read_requests_rate,
                                                          write_requests_rate=write_requests_rate)
            yield(disk,disk_rate_info)

    def inspect_vnics(self, instance):
        info = self.get_monitor(instance,'network')
        for key in info.keys():
            if key == 'code' or key == 'msg':
                continue
            device = key
            interface = virt_inspector.Interface(name=device,
                                                 mac=None,
					         fref=None,
					         parameters=None)
            rx_bytes = info[key]['b_in_total']
            rx_packets = info[key]['pack_in_total']
            tx_bytes = info[key]['b_out_total']
            tx_packets = info[key]['pack_out_total']
            stats = virt_inspector.InterfaceStats(rx_bytes=rx_bytes,
		                                  rx_packets=rx_packets,
		                                  tx_bytes=tx_bytes,
		                                  tx_packets=tx_packets)
        yield (interface,stats)

    def inspect_vnic_rates(self,instance,duration=None):
        info = self.get_monitor(instance,'network')
        for key in info.keys():
            if key == 'code' or key == 'msg':
                continue
            device = key
            rx_bytes_rate = info[key]['kb_in'] * units.Ki
            tx_bytes_rate = info[key]['kb_out'] * units.Ki
            stats = virt_inspector.InterfaceRateStats(rx_bytes_rate=rx_bytes_rate,
                                                      tx_bytes_rate=tx_bytes_rate)
            interface = virt_inspector.Interface(name=device,
                                                 mac=None,
                                                 fref=None,
                                                 parameters=None)
            yield (interface,stats)


